/*
## Cypress Image Sensor Routines for OmniVision OV5640 sensor (cy_ov5640.c)
## ===========================
##
##  Copyright Cypress Semiconductor Corporation, 2013,
##  All Rights Reserved
##  UNPUBLISHED, LICENSED SOFTWARE.
##
##  CONFIDENTIAL AND PROPRIETARY INFORMATION
##  WHICH IS THE PROPERTY OF CYPRESS.
##
##  Use of this file is governed
##  by the license agreement included in the file
##
##     <install>/license/license.txt
##
##  where <install> is the Cypress software
##  installation root directory path.
##
## ===========================
*/

#include "cy_ov5640.h"
#include <cyu3error.h>
#include <cyu3i2c.h>
#include <cyu3utils.h>

/* OV5640 Misc Macros */
#define OV5640_I2C_ADRESS               (0x3C)
#define OV5640_I2C_READ_ADDRESS         ((OV5640_I2C_ADRESS << 1) | 1)
#define OV5640_I2C_WRITE_ADDRESS        ((OV5640_I2C_ADRESS << 1))
#define OV5640_CHIP_ID                  (0x5640)

/* OV5640 Regiser Addresses */
#define OV5640_SYSTEM_CTRL0             (0x3008)
#define OV5640_CHIP_ID_HIGH_BYTE        (0x300A)
#define OV5640_CHIP_ID_LOW_BYTE         (0x300B)
#define OV5640_POLARITY_CTRL00          (0x4740)
#define OV5640_AUTOFOCUS_CMD_MAIN       (0x3022)
#define OV5640_AUTOFOCUS_CMD_ACK        (0x3023)

/* OV5640 Register MASKS */
#define OV5640_SYSTEM_RESET_MASK        (0x80)
#define OV5640_SYSTEM_POWER_DOWN_MASK   (0x40)

#define OV5640_AF_SINGLE_TRIGGER        (0x03)
#define OV5640_AF_CONT_TRIGGER          (0x04)
#define OV5640_AF_RELAUNCH_ZONE_CONFIG  (0x12)



/* Structure to store the Address/Value pair for each
 * register to be updated. */
typedef struct CyOv5640Reg
{
    uint16_t regAddr;
    uint8_t regValue;
} CyOv5640Reg_t;

typedef struct CyOv5640AFReg
{
    uint16_t baseAddress;
    uint16_t numBytes;
    uint8_t buffer[256];
} CyOv5640AFReg_t;

/* Globals */

static CyBool_t glIsValidSensor = CyFalse;

/* Configuration Settings*/
/* Autofocus Initialization Settings */
CyOv5640AFReg_t OV5640_AutoFocusConfigurationSettings [] =
{
    {0x8000, 256,
        {
            0x02, 0x0f, 0x0c, 0x02, 0x0a, 0x09, 0xc2, 0x01, 0x22, 0x22, 0x00, 0x02, 0x0e, 0xf6, 0xe5, 0x1f, 0x70, 0x72, 0xf5, 0x1e, 0xd2, 0x35, 0xff, 0xef, 0x25, 0xe0, 0x24, 0x4c, 0xf8, 0xe4, 0xf6, 0x08, 
            0xf6, 0x0f, 0xbf, 0x34, 0xf2, 0x90, 0x0e, 0x8c, 0xe4, 0x93, 0xff, 0xe5, 0x49, 0xc3, 0x9f, 0x50, 0x04, 0x7f, 0x05, 0x80, 0x02, 0x7f, 0xfb, 0x78, 0xbb, 0xa6, 0x07, 0x12, 0x0d, 0xaf, 0x40, 0x04, 
            0x7f, 0x03, 0x80, 0x02, 0x7f, 0x30, 0x78, 0xba, 0xa6, 0x07, 0xe6, 0x18, 0xf6, 0x08, 0xe6, 0x78, 0xb7, 0xf6, 0x78, 0xba, 0xe6, 0x78, 0xb8, 0xf6, 0x78, 0xbd, 0x76, 0x33, 0xe4, 0x08, 0xf6, 0x78, 
            0xb6, 0x76, 0x01, 0x75, 0x48, 0x02, 0x78, 0xb4, 0xf6, 0x08, 0xf6, 0x74, 0xff, 0x78, 0xbf, 0xf6, 0x08, 0xf6, 0x75, 0x1f, 0x01, 0x78, 0xba, 0xe6, 0x75, 0xf0, 0x05, 0xa4, 0xf5, 0x49, 0x12, 0x0a, 
            0xcf, 0xc2, 0x37, 0x22, 0x78, 0xb6, 0xe6, 0xd3, 0x94, 0x00, 0x40, 0x02, 0x16, 0x22, 0xe5, 0x1f, 0xb4, 0x05, 0x23, 0xe4, 0xf5, 0x1f, 0xc2, 0x01, 0x78, 0xb4, 0xe6, 0xfe, 0x08, 0xe6, 0xff, 0x78, 
            0x4c, 0xa6, 0x06, 0x08, 0xa6, 0x07, 0xa2, 0x37, 0xe4, 0x33, 0xf5, 0x3c, 0x90, 0x30, 0x28, 0xf0, 0x75, 0x1e, 0x10, 0xd2, 0x35, 0x22, 0xe5, 0x49, 0x75, 0xf0, 0x05, 0x84, 0x78, 0xba, 0xf6, 0x90, 
            0x0e, 0x85, 0xe4, 0x93, 0xff, 0x25, 0xe0, 0x24, 0x0a, 0xf8, 0xe6, 0xfc, 0x08, 0xe6, 0xfd, 0x78, 0xba, 0xe6, 0x25, 0xe0, 0x24, 0x4c, 0xf8, 0xa6, 0x04, 0x08, 0xa6, 0x05, 0xef, 0x12, 0x0d, 0xb6, 
            0xd3, 0x78, 0xb5, 0x96, 0xee, 0x18, 0x96, 0x40, 0x0d, 0x78, 0xba, 0xe6, 0x78, 0xb7, 0xf6, 0x78, 0xb4, 0xa6, 0x06, 0x08, 0xa6, 0x07, 0x90, 0x0e, 0x85, 0xe4, 0x93, 0x12, 0x0d, 0xb6, 0xc3, 0x78
        }
    },

    {0x8100, 256,
        {
            0xc0, 0x96, 0xee, 0x18, 0x96, 0x50, 0x0d, 0x78, 0xba, 0xe6, 0x78, 0xb8, 0xf6, 0x78, 0xbf, 0xa6, 0x06, 0x08, 0xa6, 0x07, 0x78, 0xb4, 0xe6, 0xfe, 0x08, 0xe6, 0xc3, 0x78, 0xc0, 0x96, 0xff, 0xee, 
            0x18, 0x96, 0x78, 0xc1, 0xf6, 0x08, 0xa6, 0x07, 0x90, 0x0e, 0x8e, 0xe4, 0x18, 0x12, 0x0d, 0x94, 0x40, 0x02, 0xd2, 0x37, 0x78, 0xba, 0xe6, 0x08, 0x26, 0x08, 0xf6, 0xe5, 0x1f, 0x64, 0x01, 0x70, 
            0x4a, 0xe6, 0xc3, 0x78, 0xbe, 0x12, 0x0d, 0x8a, 0x40, 0x05, 0x12, 0x0d, 0x85, 0x40, 0x39, 0x12, 0x0d, 0xad, 0x40, 0x04, 0x7f, 0xfe, 0x80, 0x02, 0x7f, 0x02, 0x78, 0xbb, 0xa6, 0x07, 0x78, 0xb7, 
            0xe6, 0x24, 0x03, 0x78, 0xbd, 0xf6, 0x78, 0xb7, 0xe6, 0x24, 0xfd, 0x78, 0xbe, 0xf6, 0x12, 0x0d, 0xad, 0x40, 0x06, 0x78, 0xbe, 0xe6, 0xff, 0x80, 0x04, 0x78, 0xbd, 0xe6, 0xff, 0x78, 0xbc, 0xa6, 
            0x07, 0x75, 0x1f, 0x02, 0x78, 0xb6, 0x76, 0x01, 0x02, 0x02, 0x4a, 0xe5, 0x1f, 0x64, 0x02, 0x60, 0x03, 0x02, 0x02, 0x2a, 0x78, 0xbc, 0xe6, 0xff, 0xc3, 0x78, 0xbe, 0x12, 0x0d, 0x8b, 0x40, 0x08, 
            0x12, 0x0d, 0x85, 0x50, 0x03, 0x02, 0x02, 0x28, 0x12, 0x0d, 0xad, 0x40, 0x04, 0x7f, 0xff, 0x80, 0x02, 0x7f, 0x01, 0x78, 0xbb, 0xa6, 0x07, 0x78, 0xb7, 0xe6, 0x04, 0x78, 0xbd, 0xf6, 0x78, 0xb7, 
            0xe6, 0x14, 0x78, 0xbe, 0xf6, 0x18, 0x12, 0x0d, 0xaf, 0x40, 0x04, 0xe6, 0xff, 0x80, 0x02, 0x7f, 0x00, 0x78, 0xbd, 0xa6, 0x07, 0xd3, 0x08, 0xe6, 0x64, 0x80, 0x94, 0x80, 0x40, 0x04, 0xe6, 0xff, 
            0x80, 0x02, 0x7f, 0x00, 0x78, 0xbe, 0xa6, 0x07, 0xc3, 0x18, 0xe6, 0x64, 0x80, 0x94, 0xb3, 0x50, 0x04, 0xe6, 0xff, 0x80, 0x02, 0x7f, 0x33, 0x78, 0xbd, 0xa6, 0x07, 0xc3, 0x08, 0xe6, 0x64, 0x80
        }
    },

    {0x8200, 256,
        {
            0x94, 0xb3, 0x50, 0x04, 0xe6, 0xff, 0x80, 0x02, 0x7f, 0x33, 0x78, 0xbe, 0xa6, 0x07, 0x12, 0x0d, 0xad, 0x40, 0x06, 0x78, 0xbe, 0xe6, 0xff, 0x80, 0x04, 0x78, 0xbd, 0xe6, 0xff, 0x78, 0xbc, 0xa6,
            0x07, 0x75, 0x1f, 0x03, 0x78, 0xb6, 0x76, 0x01, 0x80, 0x20, 0xe5, 0x1f, 0x64, 0x03, 0x70, 0x26, 0x78, 0xbc, 0xe6, 0xff, 0xc3, 0x78, 0xbe, 0x12, 0x0d, 0x8b, 0x40, 0x05, 0x12, 0x0d, 0x85, 0x40,
            0x09, 0x78, 0xb7, 0xe6, 0x78, 0xbc, 0xf6, 0x75, 0x1f, 0x04, 0x78, 0xbc, 0xe6, 0x75, 0xf0, 0x05, 0xa4, 0xf5, 0x49, 0x02, 0x0a, 0xcf, 0xe5, 0x1f, 0xb4, 0x04, 0x10, 0x90, 0x0e, 0x8d, 0xe4, 0x78,
            0xc1, 0x12, 0x0d, 0x94, 0x40, 0x02, 0xd2, 0x37, 0x75, 0x1f, 0x05, 0x22, 0x30, 0x01, 0x03, 0x02, 0x04, 0xc0, 0x30, 0x02, 0x03, 0x02, 0x04, 0xc0, 0x90, 0x51, 0xa5, 0xe0, 0x78, 0x91, 0xf6, 0xa3,
            0xe0, 0x08, 0xf6, 0xa3, 0xe0, 0x08, 0xf6, 0xe5, 0x1f, 0x70, 0x3c, 0x75, 0x1e, 0x20, 0xd2, 0x35, 0x12, 0x0c, 0x41, 0x78, 0x7c, 0xa6, 0x06, 0x08, 0xa6, 0x07, 0x78, 0x89, 0xa6, 0x09, 0x18, 0x76,
            0x01, 0x12, 0x0c, 0x22, 0x78, 0x4c, 0xa6, 0x06, 0x08, 0xa6, 0x07, 0x78, 0x89, 0xe6, 0x78, 0x6c, 0xf6, 0x75, 0x1f, 0x01, 0x78, 0x91, 0xe6, 0x78, 0x8e, 0xf6, 0x78, 0x92, 0xe6, 0x78, 0x8f, 0xf6, 
            0x78, 0x93, 0xe6, 0x78, 0x90, 0xf6, 0x22, 0x79, 0x8e, 0xe7, 0xd3, 0x78, 0x91, 0x96, 0x40, 0x05, 0xe7, 0x96, 0xff, 0x80, 0x08, 0xc3, 0x79, 0x91, 0xe7, 0x78, 0x8e, 0x96, 0xff, 0x78, 0x86, 0x76,
            0x00, 0x08, 0xa6, 0x07, 0x79, 0x8f, 0xe7, 0xd3, 0x78, 0x92, 0x96, 0x40, 0x05, 0xe7, 0x96, 0xff, 0x80, 0x08, 0xc3, 0x79, 0x92, 0xe7, 0x78, 0x8f, 0x96, 0xff, 0x12, 0x0c, 0x55, 0x79, 0x90, 0xe7
        }
    },

    {0x8300, 256,
        {
            0xd3, 0x78, 0x93, 0x96, 0x40, 0x05, 0xe7, 0x96, 0xff, 0x80, 0x08, 0xc3, 0x79, 0x93, 0xe7, 0x78, 0x90, 0x96, 0xff, 0x12, 0x0c, 0x55, 0x12, 0x0c, 0x22, 0x78, 0x88, 0xe6, 0x25, 0xe0, 0x24, 0x4c,
            0xf8, 0xa6, 0x06, 0x08, 0xa6, 0x07, 0x78, 0x88, 0xe6, 0x24, 0x6c, 0xf8, 0xa6, 0x09, 0x78, 0x88, 0xe6, 0x24, 0x01, 0xff, 0xe4, 0x33, 0xfe, 0xd3, 0xef, 0x94, 0x0f, 0xee, 0x64, 0x80, 0x94, 0x80,
            0x40, 0x04, 0x7f, 0x00, 0x80, 0x05, 0x78, 0x88, 0xe6, 0x04, 0xff, 0x78, 0x88, 0xa6, 0x07, 0xe5, 0x1f, 0xb4, 0x01, 0x0a, 0xe6, 0x60, 0x03, 0x02, 0x04, 0xc0, 0x75, 0x1f, 0x02, 0x22, 0x12, 0x0c,
            0x41, 0x78, 0x7e, 0xa6, 0x06, 0x08, 0xa6, 0x07, 0x12, 0x0c, 0x41, 0x78, 0x80, 0xa6, 0x06, 0x08, 0xa6, 0x07, 0x78, 0x6c, 0xe6, 0x78, 0x8a, 0xf6, 0x78, 0x6c, 0xe6, 0x78, 0x8b, 0xf6, 0x7f, 0x01, 
            0xef, 0x25, 0xe0, 0x24, 0x4d, 0xf9, 0xc3, 0x78, 0x7f, 0xe6, 0x97, 0x18, 0xe6, 0x19, 0x97, 0x50, 0x0a, 0x12, 0x0c, 0x49, 0x78, 0x7e, 0xa6, 0x04, 0x08, 0xa6, 0x05, 0x74, 0x6c, 0x2f, 0xf9, 0x78,
            0x8a, 0xe6, 0xc3, 0x97, 0x50, 0x08, 0x74, 0x6c, 0x2f, 0xf8, 0xe6, 0x78, 0x8a, 0xf6, 0xef, 0x25, 0xe0, 0x24, 0x4d, 0xf9, 0xd3, 0x78, 0x81, 0xe6, 0x97, 0x18, 0xe6, 0x19, 0x97, 0x40, 0x0a, 0x12,
            0x0c, 0x49, 0x78, 0x80, 0xa6, 0x04, 0x08, 0xa6, 0x05, 0x74, 0x6c, 0x2f, 0xf9, 0x78, 0x8b, 0xe6, 0xd3, 0x97, 0x40, 0x08, 0x74, 0x6c, 0x2f, 0xf8, 0xe6, 0x78, 0x8b, 0xf6, 0x0f, 0xef, 0x64, 0x10,
            0x70, 0x9e, 0xc3, 0x79, 0x7f, 0xe7, 0x78, 0x81, 0x96, 0xff, 0x19, 0xe7, 0x18, 0x96, 0x78, 0x82, 0xf6, 0x08, 0xa6, 0x07, 0xc3, 0x79, 0x8a, 0xe7, 0x78, 0x8b, 0x96, 0x08, 0xf6, 0xd3, 0x79, 0x7f
        }
    },

    {0x8400, 256,
        {
            0xe7, 0x78, 0x7d, 0x96, 0x19, 0xe7, 0x18, 0x96, 0x40, 0x05, 0x09, 0xe7, 0x08, 0x80, 0x06, 0xc3, 0x79, 0x7d, 0xe7, 0x78, 0x7f, 0x96, 0xff, 0x19, 0xe7, 0x18, 0x96, 0xfe, 0x78, 0x84, 0xa6, 0x06,
            0x08, 0xa6, 0x07, 0x79, 0x8a, 0xe7, 0xd3, 0x78, 0x89, 0x96, 0x40, 0x05, 0xe7, 0x96, 0xff, 0x80, 0x08, 0xc3, 0x79, 0x89, 0xe7, 0x78, 0x8a, 0x96, 0xff, 0x78, 0x8d, 0xa6, 0x07, 0xe5, 0x1f, 0x64,
            0x02, 0x70, 0x69, 0x90, 0x0e, 0x8a, 0x93, 0xff, 0x18, 0xe6, 0xc3, 0x9f, 0x50, 0x72, 0x12, 0x0c, 0x11, 0x12, 0x0b, 0xf6, 0x90, 0x0e, 0x87, 0x12, 0x0b, 0xff, 0x78, 0x7e, 0x12, 0x0c, 0x32, 0x7b,
            0x04, 0x12, 0x0b, 0xe4, 0xc3, 0x12, 0x06, 0x45, 0x50, 0x56, 0x90, 0x0e, 0x8b, 0xe4, 0x93, 0xff, 0x78, 0x8d, 0xe6, 0x9f, 0x40, 0x02, 0x80, 0x11, 0x90, 0x0e, 0x89, 0xe4, 0x93, 0xff, 0xd3, 0x78,
            0x87, 0xe6, 0x9f, 0x18, 0xe6, 0x94, 0x00, 0x40, 0x03, 0x75, 0x1f, 0x05, 0x12, 0x0c, 0x11, 0x12, 0x0b, 0xf6, 0x90, 0x0e, 0x88, 0x12, 0x0b, 0xff, 0x78, 0x7c, 0x12, 0x0c, 0x32, 0x7b, 0x40, 0x12,
            0x0b, 0xe4, 0xd3, 0x12, 0x06, 0x45, 0x40, 0x18, 0x75, 0x1f, 0x05, 0x22, 0xe5, 0x1f, 0xb4, 0x05, 0x0f, 0xd2, 0x01, 0xc2, 0x02, 0xe4, 0xf5, 0x1f, 0xf5, 0x1e, 0xd2, 0x35, 0xd2, 0x33, 0xd2, 0x36,
            0x22, 0xef, 0x8d, 0xf0, 0xa4, 0xa8, 0xf0, 0xcf, 0x8c, 0xf0, 0xa4, 0x28, 0xce, 0x8d, 0xf0, 0xa4, 0x2e, 0xfe, 0x22, 0xbc, 0x00, 0x0b, 0xbe, 0x00, 0x29, 0xef, 0x8d, 0xf0, 0x84, 0xff, 0xad, 0xf0,
            0x22, 0xe4, 0xcc, 0xf8, 0x75, 0xf0, 0x08, 0xef, 0x2f, 0xff, 0xee, 0x33, 0xfe, 0xec, 0x33, 0xfc, 0xee, 0x9d, 0xec, 0x98, 0x40, 0x05, 0xfc, 0xee, 0x9d, 0xfe, 0x0f, 0xd5, 0xf0, 0xe9, 0xe4, 0xce
        }
    },

    {0x8500, 256,
        {
            0xfd, 0x22, 0xed, 0xf8, 0xf5, 0xf0, 0xee, 0x84, 0x20, 0xd2, 0x1c, 0xfe, 0xad, 0xf0, 0x75, 0xf0, 0x08, 0xef, 0x2f, 0xff, 0xed, 0x33, 0xfd, 0x40, 0x07, 0x98, 0x50, 0x06, 0xd5, 0xf0, 0xf2, 0x22,
            0xc3, 0x98, 0xfd, 0x0f, 0xd5, 0xf0, 0xea, 0x22, 0xe8, 0x8f, 0xf0, 0xa4, 0xcc, 0x8b, 0xf0, 0xa4, 0x2c, 0xfc, 0xe9, 0x8e, 0xf0, 0xa4, 0x2c, 0xfc, 0x8a, 0xf0, 0xed, 0xa4, 0x2c, 0xfc, 0xea, 0x8e,
            0xf0, 0xa4, 0xcd, 0xa8, 0xf0, 0x8b, 0xf0, 0xa4, 0x2d, 0xcc, 0x38, 0x25, 0xf0, 0xfd, 0xe9, 0x8f, 0xf0, 0xa4, 0x2c, 0xcd, 0x35, 0xf0, 0xfc, 0xeb, 0x8e, 0xf0, 0xa4, 0xfe, 0xa9, 0xf0, 0xeb, 0x8f,
            0xf0, 0xa4, 0xcf, 0xc5, 0xf0, 0x2e, 0xcd, 0x39, 0xfe, 0xe4, 0x3c, 0xfc, 0xea, 0xa4, 0x2d, 0xce, 0x35, 0xf0, 0xfd, 0xe4, 0x3c, 0xfc, 0x22, 0x75, 0xf0, 0x08, 0x75, 0x82, 0x00, 0xef, 0x2f, 0xff,
            0xee, 0x33, 0xfe, 0xcd, 0x33, 0xcd, 0xcc, 0x33, 0xcc, 0xc5, 0x82, 0x33, 0xc5, 0x82, 0x9b, 0xed, 0x9a, 0xec, 0x99, 0xe5, 0x82, 0x98, 0x40, 0x0c, 0xf5, 0x82, 0xee, 0x9b, 0xfe, 0xed, 0x9a, 0xfd,
            0xec, 0x99, 0xfc, 0x0f, 0xd5, 0xf0, 0xd6, 0xe4, 0xce, 0xfb, 0xe4, 0xcd, 0xfa, 0xe4, 0xcc, 0xf9, 0xa8, 0x82, 0x22, 0xb8, 0x00, 0xc1, 0xb9, 0x00, 0x59, 0xba, 0x00, 0x2d, 0xec, 0x8b, 0xf0, 0x84,
            0xcf, 0xce, 0xcd, 0xfc, 0xe5, 0xf0, 0xcb, 0xf9, 0x78, 0x18, 0xef, 0x2f, 0xff, 0xee, 0x33, 0xfe, 0xed, 0x33, 0xfd, 0xec, 0x33, 0xfc, 0xeb, 0x33, 0xfb, 0x10, 0xd7, 0x03, 0x99, 0x40, 0x04, 0xeb,
            0x99, 0xfb, 0x0f, 0xd8, 0xe5, 0xe4, 0xf9, 0xfa, 0x22, 0x78, 0x18, 0xef, 0x2f, 0xff, 0xee, 0x33, 0xfe, 0xed, 0x33, 0xfd, 0xec, 0x33, 0xfc, 0xc9, 0x33, 0xc9, 0x10, 0xd7, 0x05, 0x9b, 0xe9, 0x9a
        }
    },

    {0x8600, 256,
        {
            0x40, 0x07, 0xec, 0x9b, 0xfc, 0xe9, 0x9a, 0xf9, 0x0f, 0xd8, 0xe0, 0xe4, 0xc9, 0xfa, 0xe4, 0xcc, 0xfb, 0x22, 0x75, 0xf0, 0x10, 0xef, 0x2f, 0xff, 0xee, 0x33, 0xfe, 0xed, 0x33, 0xfd, 0xcc, 0x33,
            0xcc, 0xc8, 0x33, 0xc8, 0x10, 0xd7, 0x07, 0x9b, 0xec, 0x9a, 0xe8, 0x99, 0x40, 0x0a, 0xed, 0x9b, 0xfd, 0xec, 0x9a, 0xfc, 0xe8, 0x99, 0xf8, 0x0f, 0xd5, 0xf0, 0xda, 0xe4, 0xcd, 0xfb, 0xe4, 0xcc,
            0xfa, 0xe4, 0xc8, 0xf9, 0x22, 0xeb, 0x9f, 0xf5, 0xf0, 0xea, 0x9e, 0x42, 0xf0, 0xe9, 0x9d, 0x42, 0xf0, 0xe8, 0x9c, 0x45, 0xf0, 0x22, 0xe8, 0x60, 0x0f, 0xef, 0xc3, 0x33, 0xff, 0xee, 0x33, 0xfe,
            0xed, 0x33, 0xfd, 0xec, 0x33, 0xfc, 0xd8, 0xf1, 0x22, 0xe4, 0x93, 0xfc, 0x74, 0x01, 0x93, 0xfd, 0x74, 0x02, 0x93, 0xfe, 0x74, 0x03, 0x93, 0xff, 0x22, 0xe6, 0xfb, 0x08, 0xe6, 0xf9, 0x08, 0xe6,
            0xfa, 0x08, 0xe6, 0xcb, 0xf8, 0x22, 0xec, 0xf6, 0x08, 0xed, 0xf6, 0x08, 0xee, 0xf6, 0x08, 0xef, 0xf6, 0x22, 0xa4, 0x25, 0x82, 0xf5, 0x82, 0xe5, 0xf0, 0x35, 0x83, 0xf5, 0x83, 0x22, 0xd0, 0x83,
            0xd0, 0x82, 0xf8, 0xe4, 0x93, 0x70, 0x12, 0x74, 0x01, 0x93, 0x70, 0x0d, 0xa3, 0xa3, 0x93, 0xf8, 0x74, 0x01, 0x93, 0xf5, 0x82, 0x88, 0x83, 0xe4, 0x73, 0x74, 0x02, 0x93, 0x68, 0x60, 0xef, 0xa3,
            0xa3, 0xa3, 0x80, 0xdf, 0x90, 0x38, 0x04, 0x78, 0x50, 0x12, 0x0b, 0xc4, 0x90, 0x38, 0x00, 0xe0, 0xfe, 0xa3, 0xe0, 0xfd, 0xed, 0xff, 0xc3, 0x12, 0x0b, 0x7d, 0x90, 0x38, 0x10, 0x12, 0x0b, 0x71,
            0x90, 0x38, 0x06, 0x78, 0x52, 0x12, 0x0b, 0xc4, 0x90, 0x38, 0x02, 0xe0, 0xfe, 0xa3, 0xe0, 0xfd, 0xed, 0xff, 0xc3, 0x12, 0x0b, 0x7d, 0x90, 0x38, 0x12, 0x12, 0x0b, 0x71, 0xa3, 0xe0, 0xb4, 0x31
        }
    },

    {0x8700, 256,
        {
            0x07, 0x78, 0x50, 0x79, 0x50, 0x12, 0x0b, 0xda, 0x90, 0x38, 0x14, 0xe0, 0xb4, 0x71, 0x15, 0x78, 0x50, 0xe6, 0xfe, 0x08, 0xe6, 0x78, 0x02, 0xce, 0xc3, 0x13, 0xce, 0x13, 0xd8, 0xf9, 0x79, 0x51,
            0xf7, 0xee, 0x19, 0xf7, 0x90, 0x38, 0x15, 0xe0, 0xb4, 0x31, 0x07, 0x78, 0x52, 0x79, 0x52, 0x12, 0x0b, 0xda, 0x90, 0x38, 0x15, 0xe0, 0xb4, 0x71, 0x15, 0x78, 0x52, 0xe6, 0xfe, 0x08, 0xe6, 0x78,
            0x02, 0xce, 0xc3, 0x13, 0xce, 0x13, 0xd8, 0xf9, 0x79, 0x53, 0xf7, 0xee, 0x19, 0xf7, 0x79, 0x50, 0x12, 0x0b, 0xac, 0x09, 0x12, 0x0b, 0xac, 0xaf, 0x45, 0x12, 0x0b, 0x62, 0x7d, 0x50, 0x12, 0x04,
            0xd3, 0x78, 0x58, 0xa6, 0x06, 0x08, 0xa6, 0x07, 0xaf, 0x43, 0x12, 0x0b, 0x62, 0x7d, 0x50, 0x12, 0x04, 0xd3, 0x78, 0x54, 0xa6, 0x06, 0x08, 0xa6, 0x07, 0xaf, 0x46, 0x78, 0x52, 0x12, 0x0b, 0x64,
            0x7d, 0x3c, 0x12, 0x04, 0xd3, 0x78, 0x5a, 0xa6, 0x06, 0x08, 0xa6, 0x07, 0xaf, 0x44, 0x7e, 0x00, 0x78, 0x52, 0x12, 0x0b, 0x66, 0x7d, 0x3c, 0x12, 0x04, 0xd3, 0x78, 0x56, 0xa6, 0x06, 0x08, 0xa6,
            0x07, 0xc3, 0x78, 0x59, 0xe6, 0x94, 0x08, 0x18, 0xe6, 0x94, 0x00, 0x50, 0x05, 0x76, 0x00, 0x08, 0x76, 0x08, 0xc3, 0x78, 0x5b, 0xe6, 0x94, 0x08, 0x18, 0xe6, 0x94, 0x00, 0x50, 0x05, 0x76, 0x00,
            0x08, 0x76, 0x08, 0x78, 0x58, 0x12, 0x0b, 0x99, 0xff, 0xd3, 0x78, 0x55, 0xe6, 0x9f, 0x18, 0xe6, 0x9e, 0x40, 0x0e, 0x78, 0x58, 0xe6, 0x13, 0xfe, 0x08, 0xe6, 0x78, 0x55, 0x12, 0x0b, 0xcf, 0x80,
            0x04, 0x7e, 0x00, 0x7f, 0x00, 0x78, 0x5c, 0x12, 0x0b, 0x91, 0xff, 0xd3, 0x78, 0x57, 0xe6, 0x9f, 0x18, 0xe6, 0x9e, 0x40, 0x0e, 0x78, 0x5a, 0xe6, 0x13, 0xfe, 0x08, 0xe6, 0x78, 0x57, 0x12, 0x0b
        }},

    {0x8800, 256,
        {
            0xcf, 0x80, 0x04, 0x7e, 0x00, 0x7f, 0x00, 0xe4, 0xfc, 0xfd, 0x78, 0x60, 0x12, 0x06, 0x86, 0x78, 0x58, 0x12, 0x0b, 0x99, 0x78, 0x55, 0x26, 0xff, 0xee, 0x18, 0x36, 0xfe, 0x78, 0x64, 0x12, 0x0b,
            0x91, 0x78, 0x57, 0x26, 0xff, 0xee, 0x18, 0x36, 0xfe, 0xe4, 0xfc, 0xfd, 0x78, 0x68, 0x12, 0x06, 0x86, 0x12, 0x0b, 0xa1, 0x78, 0x64, 0x12, 0x06, 0x79, 0xd3, 0x12, 0x06, 0x45, 0x40, 0x08, 0x12,
            0x0b, 0xa1, 0x78, 0x64, 0x12, 0x06, 0x86, 0x78, 0x52, 0x12, 0x0b, 0xa3, 0x78, 0x68, 0x12, 0x06, 0x79, 0xd3, 0x12, 0x06, 0x45, 0x40, 0x0a, 0x78, 0x52, 0x12, 0x0b, 0xa3, 0x78, 0x68, 0x12, 0x06,
            0x86, 0x78, 0x5f, 0xe6, 0x90, 0x60, 0x01, 0xf0, 0x78, 0x63, 0xe6, 0xa3, 0xf0, 0x78, 0x67, 0xe6, 0xa3, 0xf0, 0x78, 0x53, 0xe6, 0xa3, 0xf0, 0x7d, 0x01, 0x78, 0x5f, 0x12, 0x0b, 0xbc, 0x24, 0x01,
            0x12, 0x0b, 0x85, 0x78, 0x63, 0x12, 0x0b, 0xbc, 0x24, 0x02, 0x12, 0x0b, 0x85, 0x78, 0x67, 0x12, 0x0b, 0xbc, 0x24, 0x03, 0x12, 0x0b, 0x85, 0x78, 0x6b, 0x12, 0x0b, 0xbc, 0x24, 0x04, 0x12, 0x0b,
            0x85, 0x0d, 0xbd, 0x05, 0xd4, 0xc2, 0x0e, 0xc2, 0x06, 0x22, 0x85, 0x08, 0x41, 0x90, 0x30, 0x24, 0xe0, 0xf5, 0x3d, 0xa3, 0xe0, 0xf5, 0x3e, 0xa3, 0xe0, 0xf5, 0x3f, 0xa3, 0xe0, 0xf5, 0x40, 0xa3,
            0xe0, 0xf5, 0x3c, 0xd2, 0x34, 0xe5, 0x41, 0x12, 0x06, 0x9e, 0x09, 0x0d, 0x03, 0x09, 0x11, 0x04, 0x09, 0x17, 0x05, 0x09, 0x1a, 0x06, 0x09, 0x1d, 0x07, 0x09, 0x26, 0x08, 0x09, 0x37, 0x12, 0x09, 
            0x4f, 0x18, 0x09, 0x65, 0x19, 0x09, 0x3a, 0x1a, 0x09, 0x46, 0x1b, 0x09, 0x8a, 0x80, 0x09, 0x8f, 0x81, 0x09, 0xed, 0x8f, 0x09, 0xdc, 0x90, 0x09, 0xed, 0x91, 0x09, 0xed, 0x92, 0x09, 0xed, 0x93
        }},

    {0x8900, 256,
        {
            0x09, 0xed, 0x94, 0x09, 0xed, 0x98, 0x09, 0xea, 0x9f, 0x00, 0x00, 0x0a, 0x08, 0x12, 0x0e, 0x93, 0x22, 0x12, 0x0e, 0x93, 0xd2, 0x03, 0x22, 0xd2, 0x03, 0x22, 0xc2, 0x03, 0x22, 0xa2, 0x37, 0xe4,
            0x33, 0xf5, 0x3c, 0x02, 0x09, 0xed, 0xc2, 0x01, 0xc2, 0x02, 0xc2, 0x03, 0x12, 0x0c, 0xd4, 0x75, 0x1e, 0x70, 0xd2, 0x35, 0x02, 0x09, 0xed, 0x02, 0x09, 0xd7, 0x85, 0x40, 0x48, 0x85, 0x3c, 0x49,
            0x12, 0x0a, 0xcf, 0x02, 0x09, 0xed, 0x85, 0x48, 0x40, 0x85, 0x49, 0x3c, 0x02, 0x09, 0xed, 0xe4, 0xf5, 0x22, 0xf5, 0x23, 0x85, 0x40, 0x31, 0x85, 0x3f, 0x30, 0x85, 0x3e, 0x2f, 0x85, 0x3d, 0x2e,
            0x12, 0x0d, 0xc1, 0x80, 0x1f, 0x75, 0x22, 0x00, 0x75, 0x23, 0x01, 0x74, 0xff, 0xf5, 0x2d, 0xf5, 0x2c, 0xf5, 0x2b, 0xf5, 0x2a, 0x12, 0x0d, 0xc1, 0x85, 0x2d, 0x40, 0x85, 0x2c, 0x3f, 0x85, 0x2b,
            0x3e, 0x85, 0x2a, 0x3d, 0xe4, 0xf5, 0x3c, 0x02, 0x09, 0xed, 0x12, 0x0e, 0xdc, 0x80, 0x5e, 0x85, 0x3d, 0x43, 0x85, 0x3e, 0x44, 0xe5, 0x45, 0xc3, 0x13, 0xff, 0xe5, 0x43, 0xc3, 0x9f, 0x50, 0x02,
            0x8f, 0x43, 0xe5, 0x46, 0xc3, 0x13, 0xff, 0xe5, 0x44, 0xc3, 0x9f, 0x50, 0x02, 0x8f, 0x44, 0xe5, 0x45, 0xc3, 0x13, 0xff, 0xfd, 0xe5, 0x43, 0x90, 0x0e, 0x7f, 0x12, 0x0e, 0xaf, 0x40, 0x04, 0xee,
            0x9f, 0xf5, 0x43, 0xe5, 0x46, 0xc3, 0x13, 0xff, 0xfd, 0xe5, 0x44, 0x90, 0x0e, 0x80, 0x12, 0x0e, 0xaf, 0x40, 0x04, 0xee, 0x9f, 0xf5, 0x44, 0x12, 0x06, 0xc4, 0x80, 0x11, 0x85, 0x40, 0x46, 0x85,
            0x3f, 0x45, 0x85, 0x3e, 0x44, 0x85, 0x3d, 0x43, 0x80, 0x03, 0x02, 0x06, 0xc4, 0x90, 0x30, 0x24, 0xe5, 0x3d, 0xf0, 0xa3, 0xe5, 0x3e, 0xf0, 0xa3, 0xe5, 0x3f, 0xf0, 0xa3, 0xe5, 0x40, 0xf0, 0xa3
        }},

    {0x8a00, 256,
        {
            0xe5, 0x3c, 0xf0, 0x90, 0x30, 0x23, 0xe4, 0xf0, 0x22, 0xc0, 0xe0, 0xc0, 0x83, 0xc0, 0x82, 0xc0, 0xd0, 0x90, 0x3f, 0x0c, 0xe0, 0xf5, 0x32, 0xe5, 0x32, 0x30, 0xe3, 0x74, 0x30, 0x36, 0x66, 0x90,
            0x60, 0x19, 0xe0, 0xf5, 0x0a, 0xa3, 0xe0, 0xf5, 0x0b, 0x90, 0x60, 0x1d, 0xe0, 0xf5, 0x14, 0xa3, 0xe0, 0xf5, 0x15, 0x90, 0x60, 0x21, 0xe0, 0xf5, 0x0c, 0xa3, 0xe0, 0xf5, 0x0d, 0x90, 0x60, 0x29,
            0xe0, 0xf5, 0x0e, 0xa3, 0xe0, 0xf5, 0x0f, 0x90, 0x60, 0x31, 0xe0, 0xf5, 0x10, 0xa3, 0xe0, 0xf5, 0x11, 0x90, 0x60, 0x39, 0xe0, 0xf5, 0x12, 0xa3, 0xe0, 0xf5, 0x13, 0x30, 0x01, 0x06, 0x30, 0x33,
            0x03, 0xd3, 0x80, 0x01, 0xc3, 0x92, 0x09, 0x30, 0x02, 0x06, 0x30, 0x33, 0x03, 0xd3, 0x80, 0x01, 0xc3, 0x92, 0x0a, 0x30, 0x33, 0x0c, 0x30, 0x03, 0x09, 0x20, 0x02, 0x06, 0x20, 0x01, 0x03, 0xd3,
            0x80, 0x01, 0xc3, 0x92, 0x0b, 0x90, 0x30, 0x01, 0xe0, 0x44, 0x40, 0xf0, 0xe0, 0x54, 0xbf, 0xf0, 0xe5, 0x32, 0x30, 0xe1, 0x14, 0x30, 0x34, 0x11, 0x90, 0x30, 0x22, 0xe0, 0xf5, 0x08, 0xe4, 0xf0,
            0x30, 0x00, 0x03, 0xd3, 0x80, 0x01, 0xc3, 0x92, 0x08, 0xe5, 0x32, 0x30, 0xe5, 0x12, 0x90, 0x56, 0xa1, 0xe0, 0xf5, 0x09, 0x30, 0x31, 0x09, 0x30, 0x05, 0x03, 0xd3, 0x80, 0x01, 0xc3, 0x92, 0x0d,
            0x90, 0x3f, 0x0c, 0xe5, 0x32, 0xf0, 0xd0, 0xd0, 0xd0, 0x82, 0xd0, 0x83, 0xd0, 0xe0, 0x32, 0x90, 0x0e, 0x7d, 0xe4, 0x93, 0xfe, 0x74, 0x01, 0x93, 0xff, 0xc3, 0x90, 0x0e, 0x7b, 0x74, 0x01, 0x93,
            0x9f, 0xff, 0xe4, 0x93, 0x9e, 0xfe, 0xe4, 0x8f, 0x3b, 0x8e, 0x3a, 0xf5, 0x39, 0xf5, 0x38, 0xab, 0x3b, 0xaa, 0x3a, 0xa9, 0x39, 0xa8, 0x38, 0xaf, 0x49, 0xfc, 0xfd, 0xfe, 0x12, 0x05, 0x28, 0x12
        }},

    {0x8b00, 256,
        {
            0x0e, 0xc1, 0xe4, 0x7b, 0xff, 0xfa, 0xf9, 0xf8, 0x12, 0x05, 0xb3, 0x12, 0x0e, 0xc1, 0x90, 0x0e, 0x69, 0xe4, 0x12, 0x0e, 0xd6, 0x12, 0x0e, 0xc1, 0xe4, 0x85, 0x48, 0x37, 0xf5, 0x36, 0xf5, 0x35,
            0xf5, 0x34, 0xaf, 0x37, 0xae, 0x36, 0xad, 0x35, 0xac, 0x34, 0xa3, 0x12, 0x0e, 0xd6, 0x8f, 0x37, 0x8e, 0x36, 0x8d, 0x35, 0x8c, 0x34, 0xe5, 0x3b, 0x45, 0x37, 0xf5, 0x3b, 0xe5, 0x3a, 0x45, 0x36,
            0xf5, 0x3a, 0xe5, 0x39, 0x45, 0x35, 0xf5, 0x39, 0xe5, 0x38, 0x45, 0x34, 0xf5, 0x38, 0xe4, 0xf5, 0x22, 0xf5, 0x23, 0x85, 0x3b, 0x31, 0x85, 0x3a, 0x30, 0x85, 0x39, 0x2f, 0x85, 0x38, 0x2e, 0x02,
            0x0d, 0xc1, 0x78, 0x50, 0x7e, 0x00, 0xe6, 0xfc, 0x08, 0xe6, 0xfd, 0x12, 0x04, 0xc1, 0x7c, 0x00, 0x22, 0xe0, 0xa3, 0xe0, 0x75, 0xf0, 0x02, 0xa4, 0xff, 0xae, 0xf0, 0xc3, 0x08, 0xe6, 0x9f, 0xf6,
            0x18, 0xe6, 0x9e, 0xf6, 0x22, 0xff, 0xe5, 0xf0, 0x34, 0x60, 0x8f, 0x82, 0xf5, 0x83, 0xec, 0xf0, 0x22, 0xe4, 0xfc, 0xfd, 0x12, 0x06, 0x86, 0x78, 0x5a, 0xe6, 0xc3, 0x13, 0xfe, 0x08, 0xe6, 0x13,
            0x22, 0x78, 0x50, 0xe6, 0xfe, 0x08, 0xe6, 0xff, 0xe4, 0xfc, 0xfd, 0x22, 0xe7, 0xc4, 0xf8, 0x54, 0xf0, 0xc8, 0x68, 0xf7, 0x09, 0xe7, 0xc4, 0x54, 0x0f, 0x48, 0xf7, 0x22, 0xe6, 0xfc, 0xed, 0x75,
            0xf0, 0x04, 0xa4, 0x22, 0xe0, 0xfe, 0xa3, 0xe0, 0xfd, 0xee, 0xf6, 0xed, 0x08, 0xf6, 0x22, 0x13, 0xff, 0xc3, 0xe6, 0x9f, 0xff, 0x18, 0xe6, 0x9e, 0xfe, 0x22, 0xe6, 0xc3, 0x13, 0xf7, 0x08, 0xe6,
            0x13, 0x09, 0xf7, 0x22, 0xad, 0x39, 0xac, 0x38, 0xfa, 0xf9, 0xf8, 0x12, 0x05, 0x28, 0x8f, 0x3b, 0x8e, 0x3a, 0x8d, 0x39, 0x8c, 0x38, 0xab, 0x37, 0xaa, 0x36, 0xa9, 0x35, 0xa8, 0x34, 0x22, 0x93
        }},

    {0x8c00, 256,
        {
            0xff, 0xe4, 0xfc, 0xfd, 0xfe, 0x12, 0x05, 0x28, 0x8f, 0x37, 0x8e, 0x36, 0x8d, 0x35, 0x8c, 0x34, 0x22, 0x78, 0x82, 0xe6, 0xfe, 0x08, 0xe6, 0xff, 0xe4, 0x8f, 0x37, 0x8e, 0x36, 0xf5, 0x35, 0xf5,
            0x34, 0x22, 0x90, 0x0e, 0x85, 0xe4, 0x93, 0x25, 0xe0, 0x24, 0x0a, 0xf8, 0xe6, 0xfe, 0x08, 0xe6, 0xff, 0x22, 0xe6, 0xfe, 0x08, 0xe6, 0xff, 0xe4, 0x8f, 0x3b, 0x8e, 0x3a, 0xf5, 0x39, 0xf5, 0x38,
            0x22, 0x78, 0x4c, 0xe6, 0xfe, 0x08, 0xe6, 0xff, 0x22, 0xef, 0x25, 0xe0, 0x24, 0x4c, 0xf8, 0xe6, 0xfc, 0x08, 0xe6, 0xfd, 0x22, 0x78, 0x87, 0xef, 0x26, 0xf6, 0x18, 0xe4, 0x36, 0xf6, 0x22, 0x75,
            0x89, 0x03, 0x75, 0xa8, 0x01, 0x75, 0xb8, 0x04, 0x75, 0x34, 0xff, 0x75, 0x35, 0x0e, 0x75, 0x36, 0x15, 0x75, 0x37, 0x0d, 0x12, 0x0d, 0x45, 0x12, 0x00, 0x09, 0x12, 0x0e, 0xdc, 0x12, 0x00, 0x06,
            0xd2, 0x00, 0xd2, 0x34, 0xd2, 0xaf, 0x75, 0x34, 0xff, 0x75, 0x35, 0x0e, 0x75, 0x36, 0x49, 0x75, 0x37, 0x03, 0x12, 0x0d, 0x45, 0x30, 0x08, 0x09, 0xc2, 0x34, 0x12, 0x08, 0xaa, 0xc2, 0x08, 0xd2,
            0x34, 0x30, 0x0b, 0x09, 0xc2, 0x36, 0x12, 0x02, 0x6c, 0xc2, 0x0b, 0xd2, 0x36, 0x30, 0x09, 0x09, 0xc2, 0x36, 0x12, 0x00, 0x0e, 0xc2, 0x09, 0xd2, 0x36, 0x30, 0x0e, 0x03, 0x12, 0x06, 0xc4, 0x30,
            0x35, 0xd3, 0x90, 0x30, 0x29, 0xe5, 0x1e, 0xf0, 0xb4, 0x10, 0x05, 0x90, 0x30, 0x23, 0xe4, 0xf0, 0xc2, 0x35, 0x80, 0xc1, 0xe4, 0xf5, 0x49, 0x90, 0x0e, 0x77, 0x93, 0xff, 0xe4, 0x8f, 0x37, 0xf5,
            0x36, 0xf5, 0x35, 0xf5, 0x34, 0xaf, 0x37, 0xae, 0x36, 0xad, 0x35, 0xac, 0x34, 0x90, 0x0e, 0x6a, 0x12, 0x0e, 0xd6, 0x8f, 0x37, 0x8e, 0x36, 0x8d, 0x35, 0x8c, 0x34, 0x90, 0x0e, 0x72, 0x12, 0x06
        }},

    {0x8d00, 256,
        {
            0x69, 0xef, 0x45, 0x37, 0xf5, 0x37, 0xee, 0x45, 0x36, 0xf5, 0x36, 0xed, 0x45, 0x35, 0xf5, 0x35, 0xec, 0x45, 0x34, 0xf5, 0x34, 0xe4, 0xf5, 0x22, 0xf5, 0x23, 0x85, 0x37, 0x31, 0x85, 0x36, 0x30,
            0x85, 0x35, 0x2f, 0x85, 0x34, 0x2e, 0x12, 0x0d, 0xc1, 0xe4, 0xf5, 0x22, 0xf5, 0x23, 0x90, 0x0e, 0x72, 0x12, 0x0e, 0xca, 0x12, 0x0d, 0xc1, 0xe4, 0xf5, 0x22, 0xf5, 0x23, 0x90, 0x0e, 0x6e, 0x12,
            0x0e, 0xca, 0x02, 0x0d, 0xc1, 0xae, 0x35, 0xaf, 0x36, 0xe4, 0xfd, 0xed, 0xc3, 0x95, 0x37, 0x50, 0x33, 0x12, 0x0f, 0x18, 0xe4, 0x93, 0xf5, 0x38, 0x74, 0x01, 0x93, 0xf5, 0x39, 0x45, 0x38, 0x60,
            0x23, 0x85, 0x39, 0x82, 0x85, 0x38, 0x83, 0xe0, 0xfc, 0x12, 0x0f, 0x18, 0x74, 0x03, 0x93, 0x52, 0x04, 0x12, 0x0f, 0x18, 0x74, 0x02, 0x93, 0x42, 0x04, 0x85, 0x39, 0x82, 0x85, 0x38, 0x83, 0xec, 
            0xf0, 0x0d, 0x80, 0xc7, 0x22, 0x78, 0xbc, 0xe6, 0xd3, 0x08, 0xff, 0xe6, 0x64, 0x80, 0xf8, 0xef, 0x64, 0x80, 0x98, 0x22, 0x93, 0xff, 0x7e, 0x00, 0xe6, 0xfc, 0x08, 0xe6, 0xfd, 0x12, 0x04, 0xc1,
            0x78, 0xbf, 0xe6, 0xfc, 0x08, 0xe6, 0xfd, 0xd3, 0xef, 0x9d, 0xee, 0x9c, 0x22, 0x78, 0xbb, 0xd3, 0xe6, 0x64, 0x80, 0x94, 0x80, 0x22, 0x25, 0xe0, 0x24, 0x0a, 0xf8, 0xe6, 0xfe, 0x08, 0xe6, 0xff,
            0x22, 0xa2, 0xaf, 0x92, 0x32, 0xc2, 0xaf, 0xe5, 0x23, 0x45, 0x22, 0x90, 0x0e, 0x5d, 0x60, 0x0e, 0x12, 0x0d, 0xf2, 0xe0, 0xf5, 0x2c, 0x12, 0x0d, 0xef, 0xe0, 0xf5, 0x2d, 0x80, 0x0c, 0x12, 0x0d,
            0xf2, 0xe5, 0x30, 0xf0, 0x12, 0x0d, 0xef, 0xe5, 0x31, 0xf0, 0xa2, 0x32, 0x92, 0xaf, 0x22, 0x90, 0x0e, 0x5f, 0xe4, 0x93, 0xfe, 0x74, 0x01, 0x93, 0xf5, 0x82, 0x8e, 0x83, 0x22, 0x00, 0x00, 0x00
        }},

    {0x8e00, 256, 
        {
            0x11, 0x09, 0x26, 0x12, 0x31, 0x43, 0x53, 0x54, 0x45, 0x20, 0x20, 0x20, 0x20, 0x20, 0x13, 0x01, 0x10, 0x00, 0x56, 0x40, 0x1a, 0x30, 0x29, 0x7e, 0x00, 0x30, 0x04, 0x20, 0xdf, 0x30, 0x05, 0x40,
            0xbf, 0x50, 0x03, 0x00, 0xfd, 0x50, 0x27, 0x01, 0xfe, 0x60, 0x00, 0x11, 0x00, 0x3f, 0x05, 0x30, 0x00, 0x3f, 0x06, 0x22, 0x00, 0x3f, 0x01, 0x2a, 0x00, 0x3f, 0x02, 0x00, 0x00, 0x36, 0x06, 0x07, 
            0x00, 0x3f, 0x0b, 0x0f, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x30, 0x01, 0x40, 0xbf, 0x30, 0x01, 0x00, 0xbf, 0x30, 0x29, 0x70, 0x00, 0x3a, 0x00, 0x00, 0xff, 0x3a, 0x00, 0x00, 0xff, 0x36, 0x03, 0x36,
            0x02, 0x41, 0x44, 0x58, 0x20, 0x18, 0x10, 0x0a, 0x04, 0x04, 0x00, 0x03, 0xff, 0x64, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x04, 0x06, 0x00, 0x03, 0x98, 0x00, 0xcc, 0x50,
            0x3c, 0x28, 0x1e, 0x10, 0x10, 0x02, 0x00, 0x10, 0x0c, 0x10, 0x04, 0x0c, 0x6e, 0x06, 0x05, 0x00, 0xa5, 0x5a, 0x00, 0xd2, 0x01, 0xc2, 0x02, 0xe4, 0xf5, 0x1f, 0xf5, 0x1e, 0xd2, 0x35, 0xd2, 0x33,
            0xd2, 0x36, 0xd2, 0x01, 0xc2, 0x02, 0xf5, 0x1f, 0xf5, 0x1e, 0xd2, 0x35, 0xd2, 0x33, 0x22, 0x2d, 0xfd, 0xe4, 0x33, 0xfc, 0xe4, 0x93, 0xfe, 0xfb, 0xd3, 0xed, 0x9b, 0x74, 0x80, 0xf8, 0x6c, 0x98,
            0x22, 0x8f, 0x3b, 0x8e, 0x3a, 0x8d, 0x39, 0x8c, 0x38, 0x22, 0x12, 0x06, 0x69, 0x8f, 0x31, 0x8e, 0x30, 0x8d, 0x2f, 0x8c, 0x2e, 0x22, 0x93, 0xf9, 0xf8, 0x02, 0x06, 0x56, 0x90, 0x0e, 0x81, 0x12,
            0x06, 0x69, 0x8f, 0x46, 0x8e, 0x45, 0x8d, 0x44, 0x8c, 0x43, 0xd2, 0x06, 0x30, 0x06, 0x03, 0xd3, 0x80, 0x01, 0xc3, 0x92, 0x0e, 0x22, 0xc0, 0xe0, 0xc0, 0x83, 0xc0, 0x82, 0x90, 0x3f, 0x0d, 0xe0
        }},

    {0x8f00, 35,
        {
            0xf5, 0x33, 0xe5, 0x33, 0xf0, 0xd0, 0x82, 0xd0, 0x83, 0xd0, 0xe0, 0x32, 0x78, 0x7f, 0xe4, 0xf6, 0xd8, 0xfd, 0x75, 0x81, 0xcb, 0x02, 0x0c, 0x5f, 0x8f, 0x82, 0x8e, 0x83, 0x75, 0xf0, 0x04, 0xed,
            0x02, 0x06, 0x92}
    },
    {0x3022, 8, 
        {
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7F
        }
    }


};

/* Configuration Settings for 2Lane VGA 60FPS as provided by Omnivision*/
CyOv5640Reg_t OV5640_BaseConfigurationSettings [] =
{
    {0x3103, 0x11},
    {0x3008, 0x82},
    {0x3008, 0x42},
    {0x3103, 0x03},
    {0x3017, 0x00},
    {0x3018, 0x00},
    {0x3034, 0x18},
    {0x3035, 0x12},
    {0x3036, 0x38},
    {0x3037, 0x13},
    {0x3108, 0x01},
    {0x3630, 0x36},
    {0x3631, 0x0e},
    {0x3632, 0xe2},
    {0x3633, 0x12},
    {0x3621, 0xe0},
    {0x3704, 0xa0},
    {0x3703, 0x5a},
    {0x3715, 0x78},
    {0x3717, 0x01},
    {0x370b, 0x60},
    {0x3705, 0x1a},
    {0x3905, 0x02},
    {0x3906, 0x10},
    {0x3901, 0x0a},
    {0x3731, 0x12},
    {0x3600, 0x08},
    {0x3601, 0x33},
    {0x302d, 0x60},
    {0x3620, 0x52},
    {0x371b, 0x20},
    {0x471c, 0x50},
    {0x3a13, 0x43},
    {0x3a18, 0x00},
    {0x3a19, 0xf8},
    {0x3635, 0x13},
    {0x3636, 0x03},
    {0x3634, 0x40},
    {0x3622, 0x01},
    {0x3c01, 0x34},
    {0x3c04, 0x28},
    {0x3c05, 0x98},
    {0x3c06, 0x00},
    {0x3c07, 0x08},
    {0x3c08, 0x00},
    {0x3c09, 0x1c},
    {0x3c0a, 0x9c},
    {0x3c0b, 0x40},
    {0x3820, 0x41},
    {0x3821, 0x07},
    {0x3814, 0x31},
    {0x3815, 0x31},
    {0x3800, 0x00},
    {0x3801, 0x00},
    {0x3802, 0x00},
    {0x3803, 0x04},
    {0x3804, 0x0a},
    {0x3805, 0x3f},
    {0x3806, 0x07},
    {0x3807, 0x9b},
    {0x3808, 0x02},
    {0x3809, 0x80},
    {0x380a, 0x01},
    {0x380b, 0xe0},
    {0x380c, 0x07},
    {0x380d, 0x68},
    {0x380e, 0x03},
    {0x380f, 0xd8},
    {0x3810, 0x00},
    {0x3811, 0x10},
    {0x3812, 0x00},
    {0x3813, 0x06},
    {0x3618, 0x00},
    {0x3612, 0x29},
    {0x3708, 0x64},
    {0x3709, 0x52},
    {0x370c, 0x03}, /* */
    {0x3a02, 0x03},
    {0x3a03, 0xd8},
    {0x3a08, 0x01},
    {0x3a09, 0x27},
    {0x3a0a, 0x00},
    {0x3a0b, 0xf6},
    {0x3a0e, 0x03},
    {0x3a0d, 0x04},
    {0x3a14, 0x03},
    {0x3a15, 0xd8},
    {0x4001, 0x02},
    {0x4004, 0x02},
    {0x3000, 0x00},
    {0x3002, 0x1c},
    {0x3004, 0xff},
    {0x3006, 0xc3},
    {0x300e, 0x25}, /**/
    {0x302e, 0x08},
    {0x4300, 0x30},
    {0x501f, 0x00},
    {0x4713, 0x03}, /**/
    {0x4407, 0x04},
    {0x440e, 0x00},
    {0x460b, 0x35},/**/
    {0x460c, 0x22}, /**/
#if 0
    {0x4837, 0x44},
#endif
    {0x3824, 0x02},
    {0x5000, 0xa7},
    {0x5001, 0xA3},
    {0x5180, 0xff},
    {0x5181, 0xf2},
    {0x5182, 0x00},
    {0x5183, 0x14},
    {0x5184, 0x25},
    {0x5185, 0x24},
    {0x5186, 0x09},
    {0x5187, 0x09},
    {0x5188, 0x09},
    {0x5189, 0x75},
    {0x518a, 0x54},
    {0x518b, 0xe0},
    {0x518c, 0xb2},
    {0x518d, 0x42},
    {0x518e, 0x3d},
    {0x518f, 0x56},
    {0x5190, 0x46},
    {0x5191, 0xf8},
    {0x5192, 0x04},
    {0x5193, 0x70},
    {0x5194, 0xf0},
    {0x5195, 0xf0},
    {0x5196, 0x03},
    {0x5197, 0x01},
    {0x5198, 0x04},
    {0x5199, 0x12},
    {0x519a, 0x04},
    {0x519b, 0x00},
    {0x519c, 0x06},
    {0x519d, 0x82},
    {0x519e, 0x38},
    {0x5381, 0x1e},
    {0x5382, 0x5b},
    {0x5383, 0x08},
    {0x5384, 0x0a},
    {0x5385, 0x7e},
    {0x5386, 0x88},
    {0x5387, 0x7c},
    {0x5388, 0x6c},
    {0x5389, 0x10},
    {0x538a, 0x01},
    {0x538b, 0x98},
    {0x5300, 0x08},
    {0x5301, 0x30},
    {0x5302, 0x10},
    {0x5303, 0x00},
    {0x5304, 0x08},
    {0x5305, 0x30},
    {0x5306, 0x08},
    {0x5307, 0x16},
    {0x5309, 0x08},
    {0x530a, 0x30},
    {0x530b, 0x04},
    {0x530c, 0x06},
    {0x5480, 0x01},
    {0x5481, 0x08},
    {0x5482, 0x14},
    {0x5483, 0x28},
    {0x5484, 0x51},
    {0x5485, 0x65},
    {0x5486, 0x71},
    {0x5487, 0x7d},
    {0x5488, 0x87},
    {0x5489, 0x91},
    {0x548a, 0x9a},
    {0x548b, 0xaa},
    {0x548c, 0xb8},
    {0x548d, 0xcd},
    {0x548e, 0xdd},
    {0x548f, 0xea},
    {0x5490, 0x1d},
    {0x5580, 0x02},
    {0x5583, 0x40},
    {0x5584, 0x10},
    {0x5589, 0x10},
    {0x558a, 0x00},
    {0x558b, 0xf8},
    {0x5800, 0x23},
    {0x5801, 0x14},
    {0x5802, 0x0f},
    {0x5803, 0x0f},
    {0x5804, 0x12},
    {0x5805, 0x26},
    {0x5806, 0x0c},
    {0x5807, 0x08},
    {0x5808, 0x05},
    {0x5809, 0x05},
    {0x580a, 0x08},
    {0x580b, 0x0d},
    {0x580c, 0x08},
    {0x580d, 0x03},
    {0x580e, 0x00},
    {0x580f, 0x00},
    {0x5810, 0x03},
    {0x5811, 0x09},
    {0x5812, 0x07},
    {0x5813, 0x03},
    {0x5814, 0x00},
    {0x5815, 0x01},
    {0x5816, 0x03},
    {0x5817, 0x08},
    {0x5818, 0x0d},
    {0x5819, 0x08},
    {0x581a, 0x05},
    {0x581b, 0x06},
    {0x581c, 0x08},
    {0x581d, 0x0e},
    {0x581e, 0x29},
    {0x581f, 0x17},
    {0x5820, 0x11},
    {0x5821, 0x11},
    {0x5822, 0x15},
    {0x5823, 0x28},
    {0x5824, 0x46},
    {0x5825, 0x26},
    {0x5826, 0x08},
    {0x5827, 0x26},
    {0x5828, 0x64},
    {0x5829, 0x26},
    {0x582a, 0x24},
    {0x582b, 0x22},
    {0x582c, 0x24},
    {0x582d, 0x24},
    {0x582e, 0x06},
    {0x582f, 0x22},
    {0x5830, 0x40},
    {0x5831, 0x42},
    {0x5832, 0x24},
    {0x5833, 0x26},
    {0x5834, 0x24},
    {0x5835, 0x22},
    {0x5836, 0x22},
    {0x5837, 0x26},
    {0x5838, 0x44},
    {0x5839, 0x24},
    {0x583a, 0x26},
    {0x583b, 0x28},
    {0x583c, 0x42},
    {0x583d, 0xce},
    {0x5025, 0x00},
    {0x3a0f, 0x30},
    {0x3a10, 0x28},
    {0x3a1b, 0x30},
    {0x3a1e, 0x26},
    {0x3a11, 0x60},
    {0x3a1f, 0x14},
    {0x3008, 0x02},
    {0x503d, 0x00},
    {0x3031, 0x00},
    {0x0000, 0x00}
};
CyOv5640Reg_t OV5640_VGAConfigurationSettings [] =
{
    {0x3008, 0x42},
    {0x3103, 0x03},
    {0x3035, 0x12},
    {0x3036, 0x6F},
    {0x3037, 0x13},
    {0x3108, 0x01},
    {0x3820, 0x47},
    {0x3821, 0x01},
    {0x3814, 0x31},
    {0x3815, 0x31},
    {0x3800, 0x00},
    {0x3801, 0x00},
    {0x3802, 0x00},
    {0x3803, 0x04},
    {0x3804, 0x0A},
    {0x3805, 0x3F},
    {0x3806, 0x07},
    {0x3807, 0x9B},
    {0x3808, 0x02},
    {0x3809, 0x80},
    {0x380A, 0x01},
    {0x380B, 0xE0},
    {0x380C, 0x07},
    {0x380D, 0x68},
    {0x380E, 0x03},
    {0x380F, 0xD8},
    {0x3810, 0x00},
    {0x3811, 0x10},
    {0x3812, 0x00},
    {0x3813, 0x06},
    {0x3618, 0x00},
    {0x3612, 0x29},
    {0x3708, 0x64},
    {0x3709, 0x52},
    {0x370C, 0x03},
    {0x3A02, 0x03},
    {0x3A03, 0xD8},
    {0x3A08, 0x01},
    {0x3A09, 0x27},
    {0x3A0A, 0x00},
    {0x3A0B, 0xF6},
    {0x3A0E, 0x03},
    {0x3A0D, 0x04},
    {0x3A14, 0x03},
    {0x3A15, 0xD8},
    {0x300E, 0x25},
    {0x302E, 0x08},
    {0x460C, 0x22},
    {0x4827, 0x16},
    {0x3824, 0x01},
    {0x4300, 0x30},
    {0x501F, 0x00},
    {0x5001, 0xA3},
    {0x4800, 0x34},
    {0x3008, 0x02},
    {0x0000, 0x00},
    {0x0000, 0x00}
    
};
/* Configuration settings for 2Lane 720p @60fps */
CyOv5640Reg_t OV5640_720pConfigurationSettings [] = 
{
    {0x3008 ,0x42},
    {0x3103 ,0x03},
    {0x3017 ,0x00},
    {0x3018 ,0x00},
    {0x3034 ,0x18},
    {0x3035 ,0x11},
    {0x3036 ,0x54},
    {0x3037 ,0x13},
    {0x3108 ,0x01},
    {0x3820 ,0x47},
    {0x3821 ,0x01},
    {0x3814 ,0x31},
    {0x3815 ,0x31},
    {0x3800 ,0x00},
    {0x3801 ,0x00},
    {0x3802 ,0x00},
    {0x3803 ,0xFA},
    {0x3804 ,0x0A},
    {0x3805 ,0x3F},
    {0x3806 ,0x06},
    {0x3807 ,0xA9},
    {0x3808 ,0x05},
    {0x3809 ,0x00},
    {0x380A ,0x02},
    {0x380B ,0xD0},
    {0x380C ,0x07},
    {0x380D ,0x64},
    {0x380E ,0x02},
    {0x380F ,0xE4},
    {0x3810 ,0x00},
    {0x3811 ,0x10},
    {0x3812 ,0x00},
    {0x3813 ,0x04},
    {0x3618 ,0x00},
    {0x3612 ,0x29},
    {0x3708 ,0x64},
    {0x3709 ,0x52},
    {0x370C ,0x03},
    {0x3A02 ,0x02},
    {0x3A03 ,0xE4},
    {0x3A08 ,0x01},
    {0x3A09 ,0xBC},
    {0x3A0A ,0x01},
    {0x3A0B ,0x72},
    {0x3A0E ,0x01},
    {0x3A0D ,0x02},
    {0x3A14 ,0x02},
    {0x3A15 ,0xE4},
    {0x3002 ,0x1C},
    {0x3006 ,0xC3},
    {0x300E ,0x45},
    {0x4300 ,0x30},
    {0x501F ,0x00},
    {0x4713 ,0x02},
    {0x4407 ,0x04},
    {0x460B ,0x37},
    {0x460C ,0x20},
    {0x4827 ,0x16},
    {0x3824 ,0x04},
    {0x4800 ,0x34},
    {0x3008 ,0x02}



};


/* Configuration Settings for 2Lane 1080p 30FPS as provided by Omnivision*/
CyOv5640Reg_t OV5640_1080pConfigurationSettings []=
{
    {0x3008 ,0x42},
    {0x3103 ,0x03},
    {0x3017 ,0x00},
    {0x3018 ,0x00},
    {0x3034 ,0x18},
    {0x3035 ,0x11},
    {0x3036 ,0x54},
    {0x3037 ,0x13},
    {0x3108 ,0x01},
    {0x3820 ,0x46},
    {0x3821 ,0x00},
    {0x3814 ,0x11},
    {0x3815 ,0x11},
    {0x3800 ,0x01},
    {0x3801 ,0x50},
    {0x3802 ,0x01},
    {0x3803 ,0xB2},
    {0x3804 ,0x08},
    {0x3805 ,0xEF},
    {0x3806 ,0x05},
    {0x3807 ,0xF1},
    {0x3808 ,0x07},
    {0x3809 ,0x80},
    {0x380A ,0x04},
    {0x380B ,0x38},
    {0x380C ,0x09},
    {0x380D ,0xC4},
    {0x380E ,0x04},
    {0x380F ,0x60},
    {0x3810 ,0x00},
    {0x3811 ,0x10},
    {0x3812 ,0x00},
    {0x3813 ,0x04},
    {0x3618 ,0x04},
    {0x3612 ,0x2B},
    {0x3708 ,0x64},
    {0x3709 ,0x12},
    {0x370C ,0x00},
    {0x3A02 ,0x04},
    {0x3A03 ,0x60},
    {0x3A08 ,0x01},
    {0x3A09 ,0x50},
    {0x3A0A ,0x01},
    {0x3A0B ,0x18},
    {0x3A0E ,0x03},
    {0x3A0D ,0x04},
    {0x3A14 ,0x04},
    {0x3A15 ,0x60},
    {0x3002 ,0x1C},
    {0x3006 ,0xC3},
    {0x300E ,0x45},
    {0x302E ,0x08},
    {0x4300 ,0x30},
    {0x501F ,0x00},
    {0x4713 ,0x02},
    {0x4407 ,0x04},
    {0x460B ,0x37},
    {0x460C ,0x20},
    {0x4827 ,0x16},
    {0x3824 ,0x04},
    {0x4800 ,0x34},
    {0x3008 ,0x02}



};

/* Configuration Settings for 2Lane 5M 15FPS as provided by Omnivision*/
CyOv5640Reg_t OV5640_5MConfigurationSettings[] = 
{
    {0x3103 ,0x11},
    {0x3008 ,0x42},
    {0x3103 ,0x03},
    {0x3017 ,0x00},
    {0x3018 ,0x00},
    {0x3034 ,0x18},
    {0x3035 ,0x11},
    {0x3036 ,0x54},
    {0x3037 ,0x13},
    {0x3108 ,0x01},
    {0x3820 ,0x46},
    {0x3821 ,0x00},
    {0x3814 ,0x11},
    {0x3815 ,0x11},
    {0x3800 ,0x00},
    {0x3801 ,0x00},
    {0x3802 ,0x00},
    {0x3803 ,0x00},
    {0x3804 ,0x0A},
    {0x3805 ,0x3F},
    {0x3806 ,0x07},
    {0x3807 ,0x9F},
    {0x3808 ,0x0A},
    {0x3809 ,0x20},
    {0x380A ,0x07},
    {0x380B ,0x98},
    {0x380C ,0x0B},
    {0x380D ,0x1C},
    {0x380E ,0x07},
    {0x380F ,0xB0},
    {0x3810 ,0x00},
    {0x3811 ,0x10},
    {0x3812 ,0x00},
    {0x3813 ,0x04},
    {0x3618 ,0x04},
    {0x3612 ,0x2B},
    {0x3708 ,0x64},
    {0x3709 ,0x12},
    {0x370C ,0x00},
    {0x3A02 ,0x07},
    {0x3A03 ,0xB0},
    {0x3A08 ,0x01},
    {0x3A09 ,0x27},
    {0x3A0A ,0x00},
    {0x3A0B ,0xF6},
    {0x3A0E ,0x06},
    {0x3A0D ,0x08},
    {0x3A14 ,0x07},
    {0x3A15 ,0xB0},
    {0x300E ,0x45},
    {0x302E ,0x08},
    {0x4300 ,0x30},
    {0x501F ,0x00},
    {0x440E ,0x00},
    {0x4827 ,0x16},
    {0x3824 ,0x01},
    {0x5001 ,0x83},
    {0x4800 ,0x34},
    {0x3008 ,0x02}


};

CyU3PReturnStatus_t OV5640_SensorRead(uint16_t regAddr, uint8_t count, uint8_t *buf)
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;
    uint8_t cnt=0;

    for(cnt=0; cnt<3 ; cnt++)
    {
        preamble.buffer[1] = CY_U3P_GET_MSB (regAddr);
        preamble.buffer[2] = CY_U3P_GET_LSB (regAddr);
        preamble.buffer[3] = OV5640_I2C_READ_ADDRESS; /* Slave address: Read operation */
        preamble.buffer[0] = OV5640_I2C_WRITE_ADDRESS; /* Slave address: write operation */
        preamble.length = 4;
        preamble.ctrlMask = 0x0004;

        status = CyU3PI2cReceiveBytes (&preamble, buf, count,0);
        CyU3PThreadSleep(1);
        if (status == CY_U3P_SUCCESS)
        {
            break;
        }
#ifdef OV5640_DEBUG
        else
            CyU3PDebugPrint(4,"\r\nImageSensorSensorRead Failed addr=0x%x",regAddr);
#endif
    }
    return status;
}
CyU3PReturnStatus_t OV5640_SensorWrite(uint16_t regAddr, uint16_t count, uint8_t *buf)
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;
    uint8_t cnt=0;

    for(cnt=0; cnt<3 ; cnt++)
    {
        preamble.buffer[1] = CY_U3P_GET_MSB (regAddr);
        preamble.buffer[2] = CY_U3P_GET_LSB (regAddr);
        preamble.buffer[0] = OV5640_I2C_WRITE_ADDRESS; /* Slave address: write operation */
        preamble.length = 3;
        preamble.ctrlMask = 0x0000;

        status = CyU3PI2cTransmitBytes (&preamble, buf, count,0);
        CyU3PThreadSleep(1);
        if (status == CY_U3P_SUCCESS)
        {
            break;
        }
#ifdef OV5640_DEBUG
        else
            CyU3PDebugPrint(4,"\r\nImageSensorSensorWrite Failed addr=0x%x",regAddr);
#endif
    }
    return status;
}


/* Function to verify that the image sensor is the OV5640 chip 
 * Sets up flag glIsValidSensor which is to be verified before 
 * any other public calls are executed*/
CyU3PReturnStatus_t OV5640_VerifyChipId()
{

    uint8_t readBuffer [2];

    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

    readBuffer [0] = readBuffer [1] = 0;
    status = OV5640_SensorRead(OV5640_CHIP_ID_HIGH_BYTE, 2, readBuffer);
    if (status != CY_U3P_SUCCESS)
        return status;

    if (OV5640_CHIP_ID != CY_U3P_MAKEWORD(readBuffer[0], readBuffer[1]))
    {
        status = CY_U3P_ERROR_BAD_ARGUMENT;
    }
    else
    {
        glIsValidSensor = CyTrue;
    }

#ifdef OV5640_DEBUG
    CyU3PDebugPrint(4,"\r\nChip ID Read= 0x%x",CY_U3P_MAKEWORD(readBuffer[0], readBuffer[1]));
#endif
    return status;
}


CyU3PReturnStatus_t OV5640_WriteConfigurationSettings (CyOv5640Reg_t * configSettings, uint16_t configSettingsSize)
{
    uint16_t regCounter = 0;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    uint8_t rwBuffer;
#ifdef OV5640_DEBUG
    CyU3PDebugPrint (4,"\r\nWriting Configuration Settings:");
#endif

    for (regCounter = 0; ((regCounter < configSettingsSize) && (status == CY_U3P_SUCCESS)); regCounter++)
    {
#ifdef OV5640_DEBUG
        CyU3PDebugPrint (4,"\r\nRegister = 0x%x: Value = 0x%x",configSettings[regCounter].regAddr, 
                configSettings[regCounter].regValue);
#endif
        rwBuffer = configSettings[regCounter].regValue;
        status = OV5640_SensorWrite (configSettings[regCounter].regAddr, 1, &rwBuffer); 
    }

    return status;
}

/* Configure AutoFocus Logic for OV5640*/
#if 0
CyU3PReturnStatus_t OV5640_ConfigureAutofocus ()
{
    uint16_t configSize;
    if (glIsValidSensor != CyTrue)
        return CY_U3P_ERROR_NOT_SUPPORTED;


    configSize = (sizeof(OV5640_AutoFocusConfigurationSettings))/(sizeof(CyOv5640Reg_t));
    return OV5640_WriteConfigurationSettings(OV5640_AutoFocusConfigurationSettings, configSize);
}
#else
CyU3PReturnStatus_t OV5640_ConfigureAutofocus ()
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    uint8_t count = 0;   
    uint8_t rwBuffer = 0;
    if (glIsValidSensor != CyTrue)
        return CY_U3P_ERROR_NOT_SUPPORTED;
    
    rwBuffer = 0x20;

    status = OV5640_SensorWrite (0x3000, 1, &rwBuffer);
    if(status != CY_U3P_SUCCESS)
            return status;

        
    for (count = 0; count <= 0x10; count++)
    {
        status = OV5640_SensorWrite(OV5640_AutoFocusConfigurationSettings[count].baseAddress, OV5640_AutoFocusConfigurationSettings[count].numBytes, OV5640_AutoFocusConfigurationSettings[count].buffer);
        if(status != CY_U3P_SUCCESS)
            return status;
    }

    rwBuffer = 0x00;

    status = OV5640_SensorWrite (0x3000, 1, &rwBuffer);
    return status;
}

#endif
/* Configure OV5640 for 1080P @30FPS*/
CyU3PReturnStatus_t CyCx3_ImageSensor_Set_1080p()
{
    uint16_t configSize;
    if (glIsValidSensor != CyTrue)
        return CY_U3P_ERROR_NOT_SUPPORTED;

    configSize = (sizeof(OV5640_1080pConfigurationSettings))/(sizeof(CyOv5640Reg_t));
    return OV5640_WriteConfigurationSettings(OV5640_1080pConfigurationSettings, configSize);
}
/* Configure OV5640 for 720P @60FPS*/
CyU3PReturnStatus_t CyCx3_ImageSensor_Set_720p()
{
    uint16_t configSize;
    if (glIsValidSensor != CyTrue)
        return CY_U3P_ERROR_NOT_SUPPORTED;

    configSize = (sizeof(OV5640_720pConfigurationSettings))/(sizeof(CyOv5640Reg_t));
    return OV5640_WriteConfigurationSettings(OV5640_720pConfigurationSettings, configSize);
}

/* Configure OV5640 for VGA @60FPS*/
CyU3PReturnStatus_t CyCx3_ImageSensor_Set_Vga()
{
    uint16_t configSize;
    if (glIsValidSensor != CyTrue)
        return CY_U3P_ERROR_NOT_SUPPORTED;

    configSize = (sizeof(OV5640_VGAConfigurationSettings))/(sizeof(CyOv5640Reg_t));
    return OV5640_WriteConfigurationSettings(OV5640_VGAConfigurationSettings, configSize);

}

/* Configure OV5640 for VGA @60FPS*/
CyU3PReturnStatus_t OV5640_ImageSensor_Set_Base()
{
    uint16_t configSize;
    if (glIsValidSensor != CyTrue)
        return CY_U3P_ERROR_NOT_SUPPORTED;

    configSize = (sizeof(OV5640_BaseConfigurationSettings))/(sizeof(CyOv5640Reg_t));
    return OV5640_WriteConfigurationSettings(OV5640_BaseConfigurationSettings, configSize);

}
/* Configure OV5640 for 5M (2529x1944) @15FPS*/
CyU3PReturnStatus_t CyCx3_ImageSensor_Set_5M()
{
    uint16_t configSize;
    if (glIsValidSensor != CyTrue)
        return CY_U3P_ERROR_NOT_SUPPORTED;

    configSize = (sizeof(OV5640_5MConfigurationSettings))/(sizeof(CyOv5640Reg_t));
    return OV5640_WriteConfigurationSettings(OV5640_5MConfigurationSettings, configSize);

}


/* Sensor format is already set as part of the confuguration for OV5640 to YUV 
 * Do nothing in this call.
 */

CyU3PReturnStatus_t CyCx3_ImageSensor_Set_Format(CyU3PSensorStreamFormat_t format)
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    switch(format)
    {
        case SENSOR_YUV2:
            break;
        default:
            status = CY_U3P_ERROR_BAD_ARGUMENT;
            break;
    }
    return status;

}


CyU3PReturnStatus_t CyCx3_ImageSensor_Init(void)
{

#ifdef OV5640_DEBUG
    CyU3PDebugPrint(4,"\r\n OV5640 Camera Initialization");
#endif

    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

    status = OV5640_VerifyChipId();
    if (status != CY_U3P_SUCCESS)
        return status;

    status = OV5640_ImageSensor_Set_Base ();
    if (status != CY_U3P_SUCCESS)
        return status;

    status = OV5640_ConfigureAutofocus ();

    return status;
}

CyU3PReturnStatus_t CyCx3_ImageSensor_Sleep()
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    uint8_t rwBuffer = 0;

    if (glIsValidSensor != CyTrue)
        return CY_U3P_ERROR_NOT_SUPPORTED;

    status = OV5640_SensorRead(OV5640_SYSTEM_CTRL0, 1, &rwBuffer);
    if (status == CY_U3P_SUCCESS)
    {
        rwBuffer |= OV5640_SYSTEM_POWER_DOWN_MASK;
      //  status = OV5640_SensorWrite(OV5640_SYSTEM_CTRL0, 1, &rwBuffer);
    }
    return status;
}

CyU3PReturnStatus_t CyCx3_ImageSensor_Wakeup()
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    uint8_t rwBuffer = 0;

    rwBuffer = 0x21;

    if (glIsValidSensor != CyTrue)
        return CY_U3P_ERROR_NOT_SUPPORTED;

    status = OV5640_SensorWrite(OV5640_POLARITY_CTRL00, 1, &rwBuffer);
    if (status == CY_U3P_SUCCESS)
    {
        rwBuffer = 0;
        status = OV5640_SensorRead(OV5640_SYSTEM_CTRL0, 1, &rwBuffer);
        if (status == CY_U3P_SUCCESS)
        {
            rwBuffer &= (~OV5640_SYSTEM_POWER_DOWN_MASK);
            status = OV5640_SensorWrite(OV5640_SYSTEM_CTRL0, 1, &rwBuffer);
        }
    }
    return status;
}

CyU3PReturnStatus_t OV5640_TriggerAutoFocusCont()
{
    CyU3PReturnStatus_t status;

    uint8_t rwBuffer;
#if 0
    uint8_t count = 0;
#endif
    rwBuffer = 0x01;

    status = OV5640_SensorWrite(OV5640_AUTOFOCUS_CMD_ACK, 1, &rwBuffer);
    if( status == CY_U3P_SUCCESS)
    {
        rwBuffer = OV5640_AF_CONT_TRIGGER;
        status = OV5640_SensorWrite(OV5640_AUTOFOCUS_CMD_MAIN, 1, &rwBuffer);
#if 0
        if (status == CY_U3P_SUCCESS)
        {
            rwBuffer = 0x01;
            while ((count++ < 10) && (rwBuffer != 0) && (status == CY_U3P_SUCCESS))
            {
                status = OV5640_SensorRead(OV5640_AUTOFOCUS_CMD_ACK, 1, &rwBuffer);
            }
            if ((count >= 10) && (rwBuffer != 0))
            {
                status = CY_U3P_ERROR_ABORTED;
            }
        }
#endif

    }
    return status;

}

CyU3PReturnStatus_t CyCx3_ImageSensor_Trigger_Autofocus()
{
    if (glIsValidSensor != CyTrue)
        return CY_U3P_ERROR_NOT_SUPPORTED;
    return OV5640_TriggerAutoFocusCont();
}

/* Configure OV5640 for QVGA - Unsupported*/
CyU3PReturnStatus_t CyCx3_ImageSensor_Set_Qvga()
{
    return CY_U3P_ERROR_NOT_SUPPORTED;
}
